#!/usr/bin/env python3

"""The template engine used to render the data to the destination."""

from jinja2 import Environment, BaseLoader, ChoiceLoader, FunctionLoader, FileSystemLoader
from typing import Any
from urllib.parse import quote as urlquote

from .config import Context, DEFAULT_TEMPLATE_NAME
from .style import STYLESHEET_NAME
from .tree import SourceNode


"""The default template used to render Markdown data."""
DEFAULT_TEMPLATE = """
<html>
    <head>
        <title>{{ title }}</title>
        <link rel="stylesheet" href="{{ default_stylesheet }}"/>
    </head>
    <body>
        <h1>{{ title }}</h1>
        {{ text }}
    </body>
</html>
"""


def _create_loader(ctxt: Context) -> BaseLoader:
    """
    Create the loader used to find templates.

    Args:
        ctxt (Context): The context providing the configuration data for dragonglass.

    Returns:
        BaseLoader: A loader to be passed to the template engine.
    """
    return ChoiceLoader([
        FileSystemLoader(ctxt.template_dir),
        FunctionLoader(lambda n: DEFAULT_TEMPLATE if n == DEFAULT_TEMPLATE_NAME else None)
    ])


def create_template_environment(ctxt: Context) -> Environment:
    """
    Create the template environment used to render the data to the destination.

    Args:
        ctxt (Context): The context providing the configuration data for dragonglass.

    Returns:
        Environment: The new template environment.
    """
    return Environment(loader=_create_loader(ctxt))


def template_vars(node: SourceNode, ctxt: Context) -> dict[str, Any]:
    """
    Create the template variables for the specified node.

    Args:
        node (SourceNode): The node being processed.
        ctxt (Context): The context providing the configuration data for dragonglass.

    Returns:
        dict[str, Any]: The template variables to be used to render the node contents.
    """
    tvars = node.make_vars()

    if ctxt.relative_links:
        stylesheet_path = ctxt.source_dir / STYLESHEET_NAME
        rel_path = ctxt.source_dir / node.path
        tvars['default_stylesheet'] = urlquote(stylesheet_path.relative_to(rel_path.parent, walk_up=True).as_posix())
        pass
    else:
        tvars['default_stylesheet'] = urlquote(ctxt.url_prefix + STYLESHEET_NAME)
        pass

    return tvars
